#ifndef GL_H
#define GL_H

#include "fix.h"

//Either colors or textures, but not both
#define TEXTURE_SUPPORT

//Works best with 100
#define NEAR_PLANE 100

#define MATRIX_STACK_SIZE 5

//Change if you want to, but it may not work.
typedef Fix<9, int_fast32_t> GLFix;

/* Column vectors and matrices:
 * [ 0 4  8 12 ]   [x]
 * [ 1 5  9 13 ] * [y]
 * [ 2 6 10 14 ]   [z]
 * [ 3 7 11 15 ]   [1]
 */

typedef struct VERTEX { GLFix x; GLFix y; GLFix z; GLFix u; GLFix v; } VERTEX;
typedef uint16_t COLOR;

typedef struct TEXTURE { int width; int height; COLOR *bitmap; } TEXTURE;

class MATRIX {
public:
    MATRIX() { memset(data, 0, sizeof(data)); }
    GLFix data[16];
};

#define GL_COLOR_BUFFER_BIT 1<<0
#define GL_DEPTH_BUFFER_BIT 1<<1

enum GLDrawMode
{
    GL_TRIANGLES,
    GL_QUADS,
    GL_QUAD_STRIP //Not really tested
};

void set_screen(uint16_t* screenBuf);
void init_gl();
void deinit_gl();
void set_color(COLOR c);
void rotate_x(GLFix a);
void rotate_y(GLFix a);
void rotate_z(GLFix a);
void add_vertex(VERTEX* vertex);

void glLoadIdentity();
void glBegin(GLDrawMode mode);
inline void glEnd() { }
void glClear(int buffers);
void glTranslatef(GLFix x, GLFix y, GLFix z);
void glBindTexture(TEXTURE *tex);
void glTexCoord2f(GLFix nu, GLFix nv);
void glColor3f(GLFix r, GLFix g, GLFix b);
void glVertex3f(GLFix x, GLFix y, GLFix z);
void glScale3f(GLFix x, GLFix y, GLFix z);
void glPushMatrix();
void glPopMatrix();

#endif
