extern "C" {
#include <common.h>
}

#include "gl.h"
#include "fastmath.h"

//Textures taken from another project of mine, license: WTFPL
#include "environment.h"
#include "char.h"

#define HEIGHT 240
#define WIDTH 320

static uint16_t *current_screen, *screen1, *screen2;

static void render_cube(GLFix x, GLFix y, GLFix z, GLFix w, GLFix h, GLFix l, GLFix u1, GLFix u2, GLFix v1, GLFix v2)
{
    w += x;
    h += y;
    l += z;

    glBegin(GL_QUADS);

    //if(draw_front)
    {
        glTexCoord2f(u1, v2);
        glVertex3f(x, y, z);
        glTexCoord2f(u1, v1);
        glVertex3f(x, h, z);
        glTexCoord2f(u2, v1);
        glVertex3f(w, h, z);
        glTexCoord2f(u2, v2);
        glVertex3f(w, y, z);
    }

    //if(draw_back)
    {
        glTexCoord2f(u2, v2);
        glVertex3f(w, y, l);
        glTexCoord2f(u2, v1);
        glVertex3f(w, h, l);
        glTexCoord2f(u1, v1);
        glVertex3f(x, h, l);
        glTexCoord2f(u1, v2);
        glVertex3f(x, y, l);
    }

    //if(draw_left)
    {
        glTexCoord2f(u1, v2);
        glVertex3f(x, y, l);
        glTexCoord2f(u1, v1);
        glVertex3f(x, h, l);
        glTexCoord2f(u2, v1);
        glVertex3f(x, h, z);
        glTexCoord2f(u2, v2);
        glVertex3f(x, y, z);
    }

    //if(draw_right)
    {
        glTexCoord2f(u1, v2);
        glVertex3f(w, y, z);
        glTexCoord2f(u1, v1);
        glVertex3f(w, h, z);
        glTexCoord2f(u2, v1);
        glVertex3f(w, h, l);
        glTexCoord2f(u2, v2);
        glVertex3f(w, y, l);
    }

    //if(draw_top)
    {
        glTexCoord2f(u1, v2);
        glVertex3f(x, h, z);
        glTexCoord2f(u1, v1);
        glVertex3f(x, h, l);
        glTexCoord2f(u2, v1);
        glVertex3f(w, h, l);
        glTexCoord2f(u2, v2);
        glVertex3f(w, h, z);
    }

    //if(draw_bottom)
    {
        glTexCoord2f(u2, v1);
        glVertex3f(w, y, z);
        glTexCoord2f(u2, v2);
        glVertex3f(w, y, l);
        glTexCoord2f(u1, v2);
        glVertex3f(x, y, l);
        glTexCoord2f(u1, v1);
        glVertex3f(x, y, z);
    }
    glEnd();
}

int main()
{
    if(!lcd_isincolor())
    {
        show_msgbox("Error", "Your LCD hasn't got enough colors :-P");
        return EXIT_FAILURE;
    }

    uint16_t** screen_address = (uint16_t**)0xC0000010;
    uint16_t* real_screen = *screen_address;

    screen1 = (uint16_t*)malloc(WIDTH*HEIGHT*sizeof(uint16_t));
    screen2 = (uint16_t*)malloc(WIDTH*HEIGHT*sizeof(uint16_t));
    current_screen = screen1;

    init_gl();

    glLoadIdentity();

    TEXTURE screen { .width = 320, .height = 240, .bitmap = (uint16_t*)malloc(WIDTH*HEIGHT*sizeof(uint16_t)) };
    memcpy(screen.bitmap, real_screen, WIDTH*HEIGHT*2);

    GLFix x = 0, y = 400, z = -1400;
    GLFix xr = -50, yr = 5;
    GLFix incr = 1;
    GLFix angle = 0;

    while(!isKeyPressed(KEY_NSPIRE_ESC))
    {
        set_screen(current_screen);

        glColor3f(1, 1, 1); //White background
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

        glPushMatrix();

        rotate_x(GLFix(359) - xr.normalise_angle());
        rotate_y(GLFix(359) - yr.normalise_angle());
        glTranslatef(-x, -y, -z);

        if(isKeyPressed(KEY_NSPIRE_8))
            z += incr;
        if(isKeyPressed(KEY_NSPIRE_2))
            z -= incr;
        if(isKeyPressed(KEY_NSPIRE_6))
            x += incr;
        if(isKeyPressed(KEY_NSPIRE_4))
            x -= incr;
        if(isKeyPressed(KEY_NSPIRE_SHIFT))
            y -= incr;
        if(isKeyPressed(KEY_NSPIRE_SPACE))
            y += incr;

        if(isKeyPressed(KEY_NSPIRE_LEFT))
            yr += incr;
        if(isKeyPressed(KEY_NSPIRE_RIGHT))
            yr -= incr;
        if(isKeyPressed(KEY_NSPIRE_UP))
            xr += incr;
        if(isKeyPressed(KEY_NSPIRE_DOWN))
            xr -= incr;

        rotate_y(angle);

        glBindTexture(&screen);
        render_cube(0, 0, 0, 320, 240, 240, 0, 1, 0, 1);

        glBindTexture(&character);
        render_cube(0, 240, 0, 320, 240, 240, 0, 1, 0, 1);

        glBindTexture(&environment);
        render_cube(400, 240, 0, 320, 240, 240, 0, 1, 0, 1);
        render_cube(400, 0, 0, 320, 240, 240, 0.125f, 0.25f, 0.25f, 0.5f);

        if(isKeyPressed(KEY_NSPIRE_X))
        {
            angle += incr;
            angle.normalise_angle();
        }

        glPopMatrix();

        memcpy(real_screen, current_screen, 320*240*2);

        if(current_screen == screen1)
            current_screen = screen2;
        else
            current_screen = screen1;
    }

    free(screen1);
    free(screen2);

    *screen_address = real_screen;
    return EXIT_SUCCESS;
}
